@@ -1,6 +1,114 @@
+
+#' @export
 lmmpower <- function(object, ...) UseMethod("lmmpower")
 setGeneric("lmmpower")
 
+#' Sample size calculations for linear mixed models of rate of change based on
+#' lmer, lme, or gee "placebo" pilot estimates.
+#' 
+#' These functions compute sample size for linear mixed models based on the
+#' formula due to Diggle (2002) or Liu and Liang (1997).  These formulae are
+#' expressed in terms of marginal model or Generalized Estimating Equations
+#' (GEE) parameters.  These functions translate pilot mixed effect model
+#' parameters (e.g. random intercept and/or slope, fixed effects, etc.)  into
+#' marginal model parameters so that either formula can be applied to
+#' equivalent affect. Pilot estimates are assumed to be from an appropriate
+#' "placebo" group and the parameter of interest is assumed to be the rate of
+#' change over time of the outcome.
+#' 
+#' Any parameters not explicitly stated are extracted from the fitted
+#' \code{object}.
+#' 
+#' @name lmmpower
+#' @aliases lmmpower-methods lmmpower,ANY-method lmmpower,merMod-method
+#' lmmpower.default lmmpower.lme lmmpower.gee lmmpower.numeric
+#' @docType methods
+#' @param object an object returned by lme4
+#' @param n sample size per group
+#' of a mixed-effects model object to placebo data assumed to have either a
+#' random intercept, or a random intercept and random effect for time (slope);
+#' and fixed effect representing the rate of change in a placebo group.
+#' @param parameter the name or position
+#' of the rate of change parameter of interest, e.g. (\code{"time"},
+#' \code{"t"}, or \code{2} if it is the second specified fixed effect).
+#' @param pct.change the percent change
+#' in the pilot estimate of the parameter of interest (\code{beta}, the
+#' placebo/null effect)
+#' @param delta the change in the pilot estimate
+#' of the parameter of interest, computed from \code{pct.change} if left
+#' missing.
+#' @param t vector of time points
+#' @param sig.level Type I error
+#' @param power power
+#' @param alternative \code{"two.sided"} or \code{"one.sided"}
+#' @param beta pilot estimate of the placebo
+#' effect (slope or rate of change in the outcome)
+#' @param beta.CI 95\% confidence limits of
+#' the pilot estimate of beta
+#' @param delta.CI 95\% confidence limits of
+#' the effect size
+#' @param sig2.i pilot estimate of variance
+#' of random intercept
+#' @param sig2.s pilot estimate of variance
+#' of random slope
+#' @param sig2.e pilot estimate of residual
+#' variance
+#' @param cov.s.i pilot estimate of
+#' covariance of random slope and intercept
+#' @param R pilot estimate of a marginal
+#' model working correlation matrix
+#' @param method the formula to use. Defaults
+#' to \code{"diggle"} for Diggle et al (2002). Alternatively \code{"liuliang"}
+#' can be selected for Liu & Liang (1997).
+#' @param tol numerical tolerance used in root finding.
+#' @param ... other arguments
+#' @return An object of class \code{power.htest} giving the calculated sample
+#' size, N, per group and other parameters.
+#' @author Michael C. Donohue
+#' @seealso \code{\link{liu.liang.linear.power}}
+#' \code{\link{diggle.linear.power}}
+#' @references Diggle P.J., Heagerty P.J., Liang K., Zeger S.L. (2002)
+#' \emph{Analysis of longitudinal data}. Second Edition. Oxford Statistical
+#' Science Series.
+#' 
+#' Liu, G., and Liang, K. Y. (1997) Sample size calculations for studies with
+#' correlated observations. \emph{Biometrics}, 53(3), 937-47.
+#' @keywords power sample size mixed effects random effects marginal model
+#' methods
+#' @examples
+#' 
+#' \dontrun{
+#' browseVignettes(package = "longpower")
+#' }
+#' 
+#' lmmpower(delta=1.5, t = seq(0,1.5,0.25),
+#' 	sig2.i = 55, sig2.s = 24, sig2.e = 10, cov.s.i=0.8*sqrt(55)*sqrt(24), power = 0.80)
+#' lmmpower(n=208, t = seq(0,1.5,0.25),
+#' 	sig2.i = 55, sig2.s = 24, sig2.e = 10, cov.s.i=0.8*sqrt(55)*sqrt(24), power = 0.80)
+#' lmmpower(beta = 5, pct.change = 0.30, t = seq(0,1.5,0.25),
+#' 	sig2.i = 55, sig2.s = 24, sig2.e = 10, cov.s.i=0.8*sqrt(55)*sqrt(24), power = 0.80)
+#' 
+#' \dontrun{
+#' library(lme4)
+#' fm1 <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy)
+#' lmmpower(fm1, pct.change = 0.30, t = seq(0,9,1), power = 0.80)
+#' 
+#' library(nlme)
+#' fm2 <- lme(Reaction ~ Days, random=~Days|Subject, sleepstudy)
+#' lmmpower(fm2, pct.change = 0.30, t = seq(0,9,1), power = 0.80)
+#' 
+#' # random intercept only
+#' fm3 <- lme(Reaction ~ Days, random=~1|Subject, sleepstudy)
+#' lmmpower(fm3, pct.change = 0.30, t = seq(0,9,1), power = 0.80)
+#' 
+#' library(gee)
+#' fm4 <- gee(Reaction ~ Days, id = Subject,
+#'             data = sleepstudy,
+#'             corstr = "exchangeable")
+#' lmmpower(fm4, pct.change = 0.30, t = seq(0,9,1), power = 0.80)
+#' }
+#' 
+#' @export
 lmmpower.default <- function(object=NULL,
    n=NULL,
    parameter = 2,
@@ -19,6 +127,7 @@ lmmpower.default <- function(object=NULL,
    cov.s.i=NULL,
    R=NULL,
    method = c("edland", "diggle", "liuliang"),
+   tol = .Machine$double.eps^2,
    ...)
 {
 	if(sum(!sapply(list(delta, pct.change), is.null))==2) 	
@@ -52,15 +161,15 @@ lmmpower.default <- function(object=NULL,
       sig2.s=sig2.s, sig2.e=sig2.e, 
       sig.level=sig.level,
       power=power,
-      alternative=alternative,...),
+      alternative=alternative,tol=tol,...),
 	  diggle = diggle.linear.power(n=n, delta=delta, t=t, R=R, 
 	    sig.level=sig.level,
 	    power=power,
-	    alternative=alternative,...),
+	    alternative=alternative,tol=tol,...),
 	  liuliang = liu.liang.linear.power(N=N, delta=delta, u=u, v=v, R=R,
 	    sig.level=sig.level,
 	    power=power,
-	    alternative=alternative,...))
+	    alternative=alternative,tol=tol,...))
 
 	if(is.null(delta.CI)&!is.null(beta.CI)) results$delta.CI <- (results$delta/beta)*beta.CI
 	if(!is.null(beta)) results$beta <- beta
@@ -71,24 +180,24 @@ lmmpower.default <- function(object=NULL,
 		  edland = edland.linear.power(n=NULL, results$delta.CI[1], t=t, sig2.s, sig2.e, 
   	    sig.level=sig.level,
   	    power=power,
-  	    alternative=alternative,...)$n,
+  	    alternative=alternative,tol=tol,...)$n,
       diggle = diggle.linear.power(n=NULL, results$delta.CI[1], t=t, R=R, 
 		    sig.level=sig.level,
-		    power=power,...)$n,
+		    power=power,tol=tol,...)$n,
 		  liuliang = liu.liang.linear.power(N=NULL, results$delta.CI[1], u=u, v=v, R=R, 
 		    sig.level=sig.level,
-		    power=power,...)$N/2)
+		    power=power,tol=tol,...)$N/2)
 		n.lower <- switch(method,
 		  edland = edland.linear.power(n=NULL, results$delta.CI[2], t, sig2.s, sig2.e, 
         sig.level=sig.level,
         power=power,
-        alternative=alternative,...)$n,
+        alternative=alternative,tol=tol,...)$n,
       diggle = diggle.linear.power(n=NULL, results$delta.CI[2], t=t, R=R, 
 		    sig.level=sig.level,
-		    power=power,...)$n,
+		    power=power,tol=tol,...)$n,
 		  liuliang = liu.liang.linear.power(N=NULL, results$delta.CI[2], u=u, v=v, R=R, 
 		    sig.level=sig.level,
-		    power=power,...)$N/2)
+		    power=power,tol=tol,...)$N/2)
 		n.CI <- c(n.lower, n.upper)
 		if(n.CI[1]>n.CI[2]) n.CI <- n.CI[2:1]
 		results$n.CI <- n.CI 
@@ -102,6 +211,11 @@ lmmpower.default <- function(object=NULL,
 	structure(results, class = "power.longtest")
 }
 
+#' @export
+lmmpower.numeric <- lmmpower.default
+
+#' @importFrom nlme getVarCov
+#' @export
 lmmpower.lme <- function(object,
    n = NULL,
    parameter = 2,
@@ -119,6 +233,7 @@ lmmpower.lme <- function(object,
    sig2.e=NULL,
    cov.s.i=NULL,
    method = c("edland", "diggle", "liuliang"),
+   tol = .Machine$double.eps^2,
    ...)
 {
 	alternative <- match.arg(alternative)
@@ -126,7 +241,7 @@ lmmpower.lme <- function(object,
   
 	if(is.numeric(parameter)) parameter <- rownames(summary(object)$tTable)[parameter]
 	
-	tab <- nlme::getVarCov(object)
+	tab <- getVarCov(object)
 
 	if(nrow(tab)>2) stop("Too many random effects. Function is 
 	  	equipped to handle at most a random intercept and slope.")
@@ -169,9 +284,11 @@ lmmpower.lme <- function(object,
 		sig2.s = sig2.s,
 		sig2.e = sig2.e,
 		cov.s.i = cov.s.i, 
-		method = method, ...)
+		method = method,
+    tol=tol, ...)
 }
 
+#' @export
 lmmpower.gee <- function(object,
    n = NULL,
    parameter = 2,
@@ -185,6 +302,7 @@ lmmpower.gee <- function(object,
    beta.CI=NULL,
    delta.CI=NULL,
    method = c("diggle", "liuliang"),
+   tol = .Machine$double.eps^2,
    ...)
 {
 	alternative <- match.arg(alternative)
@@ -215,9 +333,12 @@ lmmpower.gee <- function(object,
 		delta.CI=delta.CI,
 		R=R,
 		t=t, 
-		method=method, ...)
+		method=method,
+    tol=tol, ...)
 }
 
+#' @importFrom lme4 VarCorr fixef getME
+#' @export
 setMethod("lmmpower", signature(object = "merMod"),
   function(object, 
    n = NULL, 
@@ -236,6 +357,7 @@ setMethod("lmmpower", signature(object = "merMod"),
    sig2.e=NULL,
    cov.s.i=NULL,
    method = c("edland", "diggle", "liuliang"),
+   tol = .Machine$double.eps^2,
    ...)
 {
   if (!(sum(sapply(list(n, delta, power, sig.level), is.null)) == 1 |
@@ -249,7 +371,7 @@ setMethod("lmmpower", signature(object = "merMod"),
   
 	if(is.numeric(parameter)) parameter <- rownames(coef(summary(object)))[parameter]
 	
-	tab <- lme4::VarCorr(object)
+	tab <- VarCorr(object)
 	if(length(tab)>1) stop("Too many grouping levels. Function is 
 	  	equipped to handle at most one grouping level.")
 	tab <- tab[[1]]
@@ -294,5 +416,6 @@ setMethod("lmmpower", signature(object = "merMod"),
 		sig2.s=sig2.s,
 		sig2.e=sig2.e,
 		cov.s.i=cov.s.i, 
-		method = method, ...)
+		method = method, 
+    tol=tol, ...)
 })
